package com.example.demo1.config;

import com.example.demo1.interceptor.LoginInterceptor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.config.annotation.*;

@Configuration
public class WebMvcConfig implements WebMvcConfigurer {
	@Autowired
	private LoginInterceptor loginInterceptor;

	@Override
	public void addCorsMappings(CorsRegistry registry) {
		registry.addMapping("/**")
				.allowedOrigins("*")
				.allowedHeaders("*")
				.allowedMethods("*")
				.allowCredentials(false)
				.exposedHeaders("access-control-allow-headers",
						"access-control-allow-methods",
						"access-control-allow-origin",
						"access-control-max-age",
						"X-Frame-Options",
						"token-status")
				.maxAge(3600);
	}

	@Override
	public void addResourceHandlers(ResourceHandlerRegistry registry) {
		registry.addResourceHandler("/static/**").addResourceLocations("classpath:/static/");

		registry.addResourceHandler("swagger-ui.html")
				.addResourceLocations("classpath:/META-INF/resources/");

		registry.addResourceHandler("/webjars/**")
				.addResourceLocations("classpath:/META-INF/resources/webjars/");
	}

	@Override
	public void addViewControllers(ViewControllerRegistry registry) {
		// registry.addViewController("/login").setViewName("login");
	}

	@Override
	public void addInterceptors(InterceptorRegistry registry) {
		registry.addInterceptor(loginInterceptor)
				.addPathPatterns("/**")
                .excludePathPatterns("/login",
						"/swagger-resources/**",
                        "/webjars/**",
                        "/swagger-ui.html",
						"/doc.html",
                        "/v2/**",
                        "/mobileApi/**",
						"/favicon.ico",
						"/error")
				.excludePathPatterns("/parkingRecord/receiveDeviceInfo")
				.excludePathPatterns("/parkingRecord/plateResult")
				.excludePathPatterns("/parkingRecord/channelPayDetail")
				.excludePathPatterns("/parkingRecord/carPayDetail")
				.excludePathPatterns("/parkingRecord/generateTemporaryCarNum")
				.excludePathPatterns("/parkingRecord/temporaryCarNumInPark")
				.excludePathPatterns("/aliPay/**")
				.excludePathPatterns("/wxPay/**")
				.excludePathPatterns("/parkingMember/createDiscount")
				.excludePathPatterns("/parkingMember/getParkingInfo")
				.excludePathPatterns("/parkingMember/addParkingMember")
				.excludePathPatterns("/parkingMemberInfo/userGetCarMember")
				.excludePathPatterns("/parkingMemberInfo/validQrCode")
				.excludePathPatterns("/parkingMemberInfo/userGetCarDetail")
				.excludePathPatterns("/parkingRecord/getTemporaryCarNum")
				.excludePathPatterns("/parkingRecord/updateSurplusParkingNumber")
				.excludePathPatterns("/parkingRecord/snapImage")
				.excludePathPatterns("/parkingRecord/receiveDeviceInfo/test")
				.excludePathPatterns("/parkingRecord/getTemporaryCarNumByOpenId")
				;
	}
}
