package com.example.demo1.interceptor;


import com.example.demo1.dto.MessageResult;
import com.example.demo1.utils.JwtUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.jsonwebtoken.Claims;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.net.URLDecoder;

@Slf4j
@Component
public class LoginInterceptor implements HandlerInterceptor {
    @Value("${jwt.secret}")
    private String jwtSecret;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        if (request.getMethod().equals("OPTIONS")){
            return true;
        }

        String token = request.getHeader("Authorization");

        if (StringUtils.isEmpty(token)){
            token = (String)request.getSession().getAttribute("token");
        }

        if (StringUtils.isEmpty(token) && !StringUtils.isEmpty(request.getParameter("token"))) {
            token = URLDecoder.decode(request.getParameter("token"),"UTF-8");
        }

        boolean result;

        try {
            Claims claims = JwtUtil.decodeTokenForClaims(jwtSecret, token);
            String subject = (String) claims.get("subject");

            if(StringUtils.isEmpty(subject)){
                log.warn("token=" + token);

                //兼容之前的token
                subject = JwtUtil.decodeTokenForSubject(jwtSecret, token);
                request.setAttribute("subject", subject);
            }
            else {
                String roles = (String) claims.get("roles");
                String permissions = (String) claims.get("permissions");

                request.setAttribute("subject", subject);
                request.setAttribute("roles", roles);
                request.setAttribute("permissions", permissions);
            }

            result = true;
        }
        catch(Exception ex){
            MessageResult<String> msgResult = new MessageResult<>();

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
            msgResult.setCode(415);

            ObjectMapper mapper = new ObjectMapper();
            String json = mapper.writeValueAsString(msgResult);

            response.setContentType("application/json");
            response.setCharacterEncoding("UTF-8");
            response.setHeader("access-control-allow-origin","*");
            response.getWriter().print(json);

            result = false;
        }

        return result;
    }
}
