package com.example.demo1.utils;

import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.security.Keys;

import java.security.Key;
import java.util.Base64;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

public class JwtUtil {
    public static String createToken(String jwtSecret, String subject, Date expiration) {
        //token有效时间天
        byte[] privateKey = Base64.getDecoder().decode(jwtSecret);

        Key key = Keys.hmacShaKeyFor(privateKey);

        Map<String,Object> extraInfo = new HashMap<>();
        extraInfo.put("subject", subject);

        String token = Jwts.builder()
//                    .setSubject(subject)
                //设置自定义claims后，setSubject值将失效
                .setClaims(extraInfo)
                .signWith(key)
                .setExpiration(expiration)
                .compact();

        return "Bearer " + token;
    }

    public static String createToken(String jwtSecret, String userId,String roles,String permissions, Date expiration) {
        //token有效时间天
        byte[] privateKey = Base64.getDecoder().decode(jwtSecret);

        Key key = Keys.hmacShaKeyFor(privateKey);

        Map<String,Object> extraInfo = new HashMap<>();
        extraInfo.put("subject", userId);
        extraInfo.put("roles", roles);
        extraInfo.put("permissions", permissions);

        String token = Jwts.builder()
//                    .setSubject(subject)
                //设置自定义claims后，setSubject值将失效
                .setClaims(extraInfo)
                .signWith(key)
                .setExpiration(expiration)
                .compact();

        return "Bearer " + token;
    }

    public static Claims decodeTokenForClaims(String jwtSecret,String token) throws Exception{
        String prefix = "Bearer ";

        if (token==null || token.length() < prefix.length()){
            throw new Exception("未传递令牌或未带前缀Bearer!");
        }

        token = token.substring(prefix.length());

        byte[] privateKey = Base64.getDecoder().decode(jwtSecret);

        Key key = Keys.hmacShaKeyFor(privateKey);

        Claims claims = Jwts.parser()
                .setSigningKey(key)
                .parseClaimsJws(token)
                .getBody();

        return claims;
    }

    public static String decodeToken(String jwtSecret,String token) throws Exception{
        Claims claims = decodeTokenForClaims(jwtSecret,token);

        String subject = (String)claims.get("subject");

        return subject;
    }

    public static String decodeTokenForSubject(String jwtSecret,String token) throws Exception{
        Claims claims = decodeTokenForClaims(jwtSecret,token);

        return claims.getSubject();
    }

    public static void main(String[] args) throws Exception {
//        String jwtSecret="WJgLLiAktNj/vCNEoz6mfAmE0btwluCTk/TnJiZOIkQ=";
//        String userId="36aeb4c5-c644-4f5a-af21-92883568b271";
//        String roles="111";
//        String permissions="222";
//        Date expiration= DateTime.now().plusHours(12).toDate();
//        String token= createToken(jwtSecret,userId,roles,permissions,expiration);
//        System.out.println(token);

        String jwtSecret="WJgLLiAktNj/vCNEoz6mfAmE0btwluCTk/TnJiZOIkQ=";
        String token ="Bearer eyJhbGciOiJIUzI1NiJ9.eyJzdWJqZWN0IjoiMzZhZWI0YzUtYzY0NC00ZjVhLWFmMjEtOTI4ODM1NjhiMjcxIiwicGVybWlzc2lvbnMiOiIyMjIiLCJyb2xlcyI6IjExMSIsImV4cCI6MTY5MDYzOTg3OX0.fItHTSxF3RUY23BsMGqIQlisMHI9QiHJyQwpzXt6kBY";
         String subJect=decodeToken(jwtSecret,token);
        System.out.println(subJect);
        //Bearer eyJhbGciOiJIUzI1NiJ9.eyJzdWJqZWN0IjoiMzZhZWI0YzUtYzY0NC00ZjVhLWFmMjEtOTI4ODM1NjhiMjcxIiwicGVybWlzc2lvbnMiOiIyMjIiLCJyb2xlcyI6IjExMSIsImV4cCI6MTY5MDQ4OTkyM30.TFRxRggZ3BoNn-D1Zxx0aflv8B4_wEF8eCyMhU0b094
        //Bearer eyJhbGciOiJIUzI1NiJ9.eyJzdWJqZWN0IjoiMzZhZWI0YzUtYzY0NC00ZjVhLWFmMjEtOTI4ODM1NjhiMjcxIiwicGVybWlzc2lvbnMiOiIyMjIiLCJyb2xlcyI6IjExMSIsImV4cCI6MTY5MDYzOTg3OX0.fItHTSxF3RUY23BsMGqIQlisMHI9QiHJyQwpzXt6kBY
    }
}
